require File.join(File.dirname(__FILE__), '..', 'lib', 'setti')
require 'spec_helper'

describe Setti::Settings do
  subject { Setti::Settings.new(File.join(File.dirname(__FILE__), 'sample.json')) }

  it 'does load sample config file' do
    expect { subject }.to_not raise_error
  end

  it 'does provide dot accessors' do
    expect(subject['numbers.integers.bad']).to eql('80')
    expect(subject['numbers.integers.good']).to eql(80)
  end

  it 'does provide hash accessors' do
    expect(subject['numbers']['integers']['bad']).to eql('80')
    expect(subject['numbers']['integers']['good']).to eql(80)

    # Symbols and also recursive
    expect(subject[:numbers][:integers][:bad]).to eql('80')
    expect(subject[:numbers][:integers][:good]).to eql(80)
  end

  it 'does validate numbers' do
    expect { subject.validate 'numbers.integers.bad',  :number }.to raise_error
    expect { subject.validate 'numbers.integers.good', :number }.to_not raise_error
    expect { subject.validate 'numbers.floats.bad',    :float }.to raise_error
    expect { subject.validate 'numbers.floats.good',   :float }.to_not raise_error
  end

  it 'does validate regexes' do
    expect { subject.validate 'regexes.good', :regex, /^\d[a-zA-Z]{3}$/ }.to_not raise_error
    expect { subject.validate 'regexes.bad',  :regex, /^\d[a-zA-Z]{3}$/ }.to raise_error
  end

  it 'does validate procs' do
    expect { subject.validate('procs.true') { |val| val } }.to_not raise_error
    expect { subject.validate('procs.true') { |val| !val } }.to raise_error
  end

  context 'Config. file missing' do
    subject { Setti::Settings.new('foo.json') }

    it 'does raise a ENOENT error' do
      expect { subject }.to raise_error
    end
  end
end
